==> m4e_sdk-0.0.1.tar.gz: test/services/m4e_transaction_test.dart <==
import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:m4e_sdk/src/core/core.dart';
import 'package:m4e_sdk/src/core/mocks/placeholder.dart' as placeholder;
import 'package:m4e_sdk/src/data/api/apis.dart';
import 'package:m4e_sdk/src/data/services/services.dart';
import 'package:m4e_sdk/src/models/models.dart';
import 'package:m4e_sdk/src/m4e_forms/m4e_forms.dart';
import 'package:matcher/matcher.dart';
import 'package:mockito/mockito.dart';
import 'package:uuid_type/uuid_type.dart';
// import 'package:matcher/matcher.dart';

class MockTransactionApi extends Mock implements TransactionApi {}

class MockDataConnectionChecker extends Mock implements DataConnectionChecker {}

main() {
  M4eTransaction _m4eTransaction;

  MockTransactionApi _mockTransactionApi;
  MockDataConnectionChecker _mockDataConnectionChecker;

  setUp(() {
    _mockTransactionApi = MockTransactionApi();
    _mockDataConnectionChecker = MockDataConnectionChecker();

    _m4eTransaction = M4eTransaction(
      transactionApi: _mockTransactionApi,
      connectionChecker: _mockDataConnectionChecker,
    );
  });

  final _fromUuId =
      UniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');
  final _toUuId =
      UniqueId.fromUniqueString('9e8b5ecc-9f7e-4e06-ac3d-05b11a91da88');

  final _tTransaction = M4eTransactionForm(
    from: Wallet(
      id: _fromUuId,
      walletInfo: WalletInfo(label: 'W1', currency: 'GHS'),
    ),
    to: Wallet(
      id: _toUuId,
      walletInfo: WalletInfo(label: 'W2', currency: 'GHS'),
    ),
    amount: Amount(currency: 'GHS', value: 50.0),
    at: DateTime.now(),
    meta: TransactionMeta(
        callbackUrl: 'www.google.com', purpose: 'Payment', data: '...'),
  );

  final _tTransactionResult = TransactionResult(
    from: _fromUuId,
    to: _toUuId,
    amount: Amount(
      currency: 'GHC',
      value: 50.0,
    ),
    at: DateTime.now(),
    txType: 'CASH_IN',
  );

  final _userId =
      UniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');

  final _transactionLogs = [
    TransactionLog(
      from: Wallet(
        id: _fromUuId,
        walletInfo: WalletInfo(label: 'W1', currency: 'GHS'),
      ),
      to: Wallet(
        id: _toUuId,
        walletInfo: WalletInfo(label: 'W2', currency: 'GHS'),
      ),
      amount: Amount(currency: 'GHS', value: 50.0),
      at: DateTime.now(),
      type: TransactionType.cashIn,
    ),
  ];

  group('m4e-transaction-test makeTransaction |', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);
      when(_mockTransactionApi.makeTransaction(any))
          .thenAnswer((_) async => _tTransactionResult);

      // act
      await _m4eTransaction.makeTransaction(_tTransaction);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected |', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should return Unit on success call', () async {
        // arrange
        when(_mockTransactionApi.makeTransaction(any))
            .thenAnswer((_) async => _tTransactionResult);

        // act
        final _result = await _m4eTransaction.makeTransaction(_tTransaction);

        // assert
        expect(_result, equals(UnitImpl()));
      });

      test('should get Transaction from TransactionApi on success call',
          () async {
        // arrange
        when(_mockTransactionApi.makeTransaction(any))
            .thenAnswer((_) async => _tTransactionResult);

        // act
        await _m4eTransaction.makeTransaction(_tTransaction);

        // assert
        verify(_mockTransactionApi.makeTransaction(_tTransaction));
      });
    });

    group('isDisconnected |', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw [ServerException] with [NO_INTERNET_CONNECTION] error code',
          () async {
        // arrange
        when(_mockTransactionApi.makeTransaction(any))
            .thenThrow(ExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eTransaction.makeTransaction;

        // assert
        expect(() => _call(_tTransaction),
            throwsA(TypeMatcher<ServerException>()));
      });
    });
  });

  group('getUserTransactionLogs | ', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);
      when(_mockTransactionApi.makeTransaction(any))
          .thenAnswer((_) async => _tTransactionResult);

      // act
      await _m4eTransaction.getUserTransactionLogs(_userId);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test(
          'should call TransactionApi to get user-transaction-logs with userId',
          () async {
        // arrange
        when(_mockTransactionApi.getUserTransactionLogs(any)).thenAnswer(
            (_) async => TransactionLogList(transactionLogs: _transactionLogs));

        // act
        await _m4eTransaction.getUserTransactionLogs(_userId);

        // assert
        verify(_mockTransactionApi.getUserTransactionLogs(_userId));
      });

      test('should return TransactionLogList on success', () async {
        // arrange
        when(_mockTransactionApi.getUserTransactionLogs(any)).thenAnswer(
            (_) async => TransactionLogList(transactionLogs: _transactionLogs));

        // act
        final _result = await _m4eTransaction.getUserTransactionLogs(_userId);

        // assert
        verify(_mockTransactionApi.getUserTransactionLogs(_userId));
        expect(_result,
            equals(TransactionLogList(transactionLogs: _transactionLogs)));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);

        test(
            'should throw a ServerException when no internet connection is detected',
            () async {
          // arrange
          when(_mockTransactionApi.getUserTransactionLogs(any))
              .thenThrow(ExceptionMessages.kNoInternetConnectionException);

          // act
          final _call = _m4eTransaction.getUserTransactionLogs;

          // assert
          expect(() => _call(_userId), throwsA(TypeMatcher<ServerException>()));
        });
      });
    });
  });
}

==> m4e_sdk-0.0.3-dev.1.tar.gz: test/services/m4e_transaction_test.dart <==
import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:m4e_sdk/src/core/core.dart';
import 'package:m4e_sdk/src/data/api/apis.dart';
import 'package:m4e_sdk/src/data/services/services.dart';
import 'package:m4e_sdk/src/m4e_forms/m4e_forms.dart';
import 'package:m4e_sdk/src/models/models.dart';
import 'package:matcher/matcher.dart';
import 'package:mockito/mockito.dart';
// import 'package:matcher/matcher.dart';

class MockTransactionApi extends Mock implements TransactionApi {}

class MockDataConnectionChecker extends Mock implements DataConnectionChecker {}

main() {
  M4eTransactionService _m4eTransaction;

  MockTransactionApi _mockTransactionApi;
  MockDataConnectionChecker _mockDataConnectionChecker;

  setUp(() {
    _mockTransactionApi = MockTransactionApi();
    _mockDataConnectionChecker = MockDataConnectionChecker();

    _m4eTransaction = M4eTransactionService(
      transactionApi: _mockTransactionApi,
      connectionChecker: _mockDataConnectionChecker,
    );
  });

  final _fromUuId =
      UniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');
  final _toUuId =
      UniqueId.fromUniqueString('9e8b5ecc-9f7e-4e06-ac3d-05b11a91da88');

  final _tTransaction = M4eTransactionForm(
    from: M4eWallet(
      id: _fromUuId,
      walletInfo: M4eWalletInfo(label: 'W1', currency: 'GHS'),
    ),
    to: M4eWallet(
      id: _toUuId,
      walletInfo: M4eWalletInfo(label: 'W2', currency: 'GHS'),
    ),
    amount: M4eAmount(currency: 'GHS', value: 50.0),
    at: DateTime.now(),
    meta: M4eTransactionMeta(
        callbackUrl: 'www.google.com', purpose: 'Payment', data: '...'),
  );

  final _tTransactionResult = M4eTransactionResult(
    from: _fromUuId,
    to: _toUuId,
    amount: M4eAmount(
      currency: 'GHC',
      value: 50.0,
    ),
    at: DateTime.now(),
    txType: 'CASH_IN',
  );

  final _userId =
      UniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');

  final _transactionLogs = [
    M4eTransactionLog(
      from: M4eWallet(
        id: _fromUuId,
        walletInfo: M4eWalletInfo(label: 'W1', currency: 'GHS'),
      ),
      to: M4eWallet(
        id: _toUuId,
        walletInfo: M4eWalletInfo(label: 'W2', currency: 'GHS'),
      ),
      amount: M4eAmount(currency: 'GHS', value: 50.0),
      at: DateTime.now(),
      type: M4eTransactionType.cashIn,
    ),
  ];

  group('m4e-transaction-test makeTransaction |', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);
      when(_mockTransactionApi.initiateTransaction(any))
          .thenAnswer((_) async => _tTransactionResult);

      // act
      await _m4eTransaction.initiateTransaction(_tTransaction);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected |', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should return Unit on success call', () async {
        // arrange
        when(_mockTransactionApi.initiateTransaction(any))
            .thenAnswer((_) async => _tTransactionResult);

        // act
        final _result =
            await _m4eTransaction.initiateTransaction(_tTransaction);

        // assert
        expect(_result, equals(UnitImpl()));
      });

      test('should get Transaction from TransactionApi on success call',
          () async {
        // arrange
        when(_mockTransactionApi.initiateTransaction(any))
            .thenAnswer((_) async => _tTransactionResult);

        // act
        await _m4eTransaction.initiateTransaction(_tTransaction);

        // assert
        verify(_mockTransactionApi.initiateTransaction(_tTransaction));
      });
    });

    group('isDisconnected |', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw [ServerException] with [NO_INTERNET_CONNECTION] error code',
          () async {
        // arrange
        when(_mockTransactionApi.initiateTransaction(any))
            .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eTransaction.initiateTransaction;

        // assert
        expect(() => _call(_tTransaction),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });
  });

  group('getUserTransactionLogs | ', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);
      when(_mockTransactionApi.initiateTransaction(any))
          .thenAnswer((_) async => _tTransactionResult);

      // act
      await _m4eTransaction.getUserTransactionLogs(_userId);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test(
          'should call TransactionApi to get user-transaction-logs with userId',
          () async {
        // arrange
        when(_mockTransactionApi.getUserTransactionLogs(any)).thenAnswer(
            (_) async =>
                M4eTransactionLogList(transactionLogs: _transactionLogs));

        // act
        await _m4eTransaction.getUserTransactionLogs(_userId);

        // assert
        verify(_mockTransactionApi.getUserTransactionLogs(_userId));
      });

      test('should return TransactionLogList on success', () async {
        // arrange
        when(_mockTransactionApi.getUserTransactionLogs(any)).thenAnswer(
            (_) async =>
                M4eTransactionLogList(transactionLogs: _transactionLogs));

        // act
        final _result = await _m4eTransaction.getUserTransactionLogs(_userId);

        // assert
        verify(_mockTransactionApi.getUserTransactionLogs(_userId));
        expect(_result,
            equals(M4eTransactionLogList(transactionLogs: _transactionLogs)));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);

        test(
            'should throw a ServerException when no internet connection is detected',
            () async {
          // arrange
          when(_mockTransactionApi.getUserTransactionLogs(any))
              .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

          // act
          final _call = _m4eTransaction.getUserTransactionLogs;

          // assert
          expect(
              () => _call(_userId), throwsA(TypeMatcher<M4eServerException>()));
        });
      });
    });
  });
}
